package de.avat.tools.tags;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.util.Iterator;
import java.util.List;

/**
 * Abstract base class for {@link TagManager}
 */
public abstract class AbstractTagManager<T> implements TagManager<T> {
  private TaggableFactory<T> taggableFactory;

  /**
   * Create a new tag manager.
   * The {@link #taggableFactory} has to be set manually when this constructor is used.
   */
  protected AbstractTagManager() {
  }

  /**
   * Create a new abstract tag manager with the given factory
   *
   * @param taggableFactory the factory that is used to create the taggables
   */
  protected AbstractTagManager( @NotNull TaggableFactory<T> taggableFactory ) {
    this.taggableFactory = taggableFactory;
  }

  @NotNull
  public Tag getTag( @NotNull @NonNls String description ) {
    try {
      return findTag( description );
    } catch ( NotFoundException ignore ) {
      return createTag( description );
    }
  }

  /**
   * Creates a tag for the given description
   *
   * @param description the description
   * @return the newly created tag for the given description
   */
  @NotNull
  protected abstract Tag createTag( @NotNull @NonNls String description );

  @NotNull
  public Taggable getTaggable( @NotNull T o ) {
    try {
      return findTaggable( o );
    } catch ( NotFoundException ignore ) {
      return createTaggable( o );
    }
  }

  @NotNull
  public String getTagsAsString( @NotNull T o ) {
    List<? extends Tag> tags = getTaggable( o ).getTags();
    StringBuilder s = new StringBuilder();
    for ( Iterator<? extends Tag> it = tags.iterator(); it.hasNext(); ) {
      Tag tag = it.next();
      s.append( tag.getDescription() );
      if ( it.hasNext() ) {
        s.append( ", " );
      }
    }

    return s.toString();
  }

  /**
   * Creates the taggable for the given object
   *
   * @param o th eobject
   * @return the taggable
   */
  @NotNull
  protected final Taggable createTaggable( @NotNull T o ) {
    return getTaggableFactory().createTaggable( o );
  }

  public void addTag( @NotNull T object, @NotNull String description ) {
    getTaggable( object ).addTag( getTag( description ) );
  }

  public void removeTag( @NotNull T object, @NotNull String description ) {
    getTaggable( object ).removeTag( getTag( description ) );
  }

  /**
   * Remove the tag.
   * Handle with care!
   *
   * @param description the description
   */
  public void removeTag( @NonNls @NotNull String description ) throws NotFoundException {
    removeTag( findTag( description ) );
  }

  @NotNull
  public Tagged findTagged( @NotNull T o ) throws NotFoundException {
    return findTaggable( o );
  }

  @NotNull
  public TaggableFactory<T> getTaggableFactory() {
    if ( taggableFactory == null ) {
      throw new IllegalStateException( "TaggableFactory not initialized properly" );
    }
    return taggableFactory;
  }

  public void setTaggableFactory( @NotNull TaggableFactory<T> taggableFactory ) {
    if ( this.taggableFactory != null ) {
      throw new IllegalStateException( "TaggableFactory still set" );
    }
    this.taggableFactory = taggableFactory;
  }
}
