package de.avat.tools.tags;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.lang.ref.WeakReference;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * A abstract implementation of {@link TagManager} that holds the tags in memory.
 * This implementation can be used when the instance of the tag itself is not important.
 */
public abstract class MemoryTagManager<T> extends AbstractTagManager<T> {
  private final TagSet<MemoryTagManager<T>> tags = new TagSet<MemoryTagManager<T>>( this );
  protected final Map<T, WeakReference<Taggable>> store = new WeakHashMap<T, WeakReference<Taggable>>();

  protected MemoryTagManager() {
  }

  @NotNull
  public Taggable findTaggable( @NotNull T o ) throws NotFoundException {
    WeakReference<Taggable> weakReference = store.get( o );
    if ( weakReference == null ) {
      throw new NotFoundException();
    }
    Taggable taggable = weakReference.get();
    if ( taggable == null ) {
      throw new NotFoundException();
    }
    return taggable;
  }

  @NotNull
  protected T getObject( @NotNull Taggable taggable ) {
    for ( Map.Entry<T, WeakReference<Taggable>> entry : store.entrySet() ) {
      if ( entry.getValue().get() == taggable ) {
        return entry.getKey();
      }
    }
    throw new IllegalArgumentException( "No object found for " + taggable );
  }


  public boolean addTag( @NotNull Tag tag ) {
    return tags.addTag( tag );
  }

  public boolean removeTag( @NotNull Tag tag ) {
    return tags.removeTag( tag );
  }

  @NotNull
  public List<? extends Tag> getTags() {
    return tags.getTags();
  }

  @Override
  @NotNull
  protected Tag createTag( @NotNull @NonNls String description ) {
    Tag newTag = new Tag( description );
    tags.addTag( newTag );
    return newTag;
  }

  @NotNull
  public Tag findTag( @NonNls @NotNull String description ) throws NotFoundException {
    for ( Tag tag : tags.getTags() ) {
      if ( tag.getDescription().equals( description ) ) {
        return tag;
      }
    }
    throw new NotFoundException( "No tag found with description " + description );
  }

  public void addTagChangeListener( @NotNull TagChangeListener listener ) {
    tags.addTagChangeListener( listener );
  }

  public void removeTagChangeListener( @NotNull TagChangeListener listener ) {
    tags.removeTagChangeListener( listener );
  }

  /**
   * Returns the store size (for internal use only)
   *
   * @return the store size
   */
  int getStoreSize() {
    return store.size();
  }
}
