package de.avat.tools.tags;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A tag set is a "bridge" between a availableTags user object and the tags itself
 */
public class TagSet<S> implements Taggable {
  private final TagChangeSupport tagChangeSupport = new TagChangeSupport();
  private final List<Tag> tags = new ArrayList<Tag>();
  private S source;

  /**
   * ID field for persistence
   */
  @Deprecated
  private int id;

  /**
   * Hibernate constructor
   */
  @Deprecated
  private TagSet() {
  }

  /**
   * Create a new TagSet
   *
   * @param source the source object
   */
  public TagSet( @NotNull S source ) {
    setSource( source );
  }

  /**
   * Sets the source
   *
   * @param source the source
   */
  protected void setSource( @NotNull S source ) {
    //noinspection ObjectEquality
    if ( this.source == source ) {
      return;
    }
    this.source = source;
    tagChangeSupport.setSource( source );
  }

  /**
   * Sets the tags
   *
   * @param tags the tags
   */
  public void setTags( @NotNull Tag... tags ) {
    this.tags.clear();
    for ( Tag tag : tags ) {
      addTag( tag );
    }
  }

  public boolean addTag( @NotNull Tag tag ) {
    if ( tags.contains( tag ) ) {
      return false;
    } else {
      tags.add( tag );
      tagChangeSupport.notifyTagAdded( tag, tags.size() - 1 );
      return true;
    }
  }

  @NotNull
  public List<? extends Tag> getTags() {
    return Collections.unmodifiableList( tags );
  }

  /**
   * Returns the source for the set
   *
   * @return the source
   */
  @NotNull
  public S getSource() {
    if ( source == null ) {
      throw new IllegalStateException( "No source set" );
    }
    return source;
  }

  public boolean removeTag( @NotNull Tag tag ) {
    int index = tags.indexOf( tag );
    if ( index > -1 ) {
      tags.remove( index );
      tagChangeSupport.notifyTagRemoved( tag, index );
      return true;
    } else {
      return false;
    }
  }

  public void addTagChangeListener( @NotNull TagChangeListener listener ) {
    tagChangeSupport.addTagChangeListener( listener );
  }

  public void removeTagChangeListener( @NotNull TagChangeListener listener ) {
    tagChangeSupport.removeTagChangeListener( listener );
  }
}
